@@ -1,7 +1,7 @@
 #line 2 "edit-distance.h.tmpl"
 #ifndef EDIT_DISTANCE_CHAR_TRANS_H
 #define EDIT_DISTANCE_CHAR_TRANS_H
-#line 1 "declaration"
+#line 5 "edit-distance-char-trans.h"
 int distance_char_trans (
                     text_fuzzy_t * tf)
 ;
