@@ -1,7 +1,7 @@
 #line 2 "edit-distance.h.tmpl"
 #ifndef EDIT_DISTANCE_INT_TRANS_H
 #define EDIT_DISTANCE_INT_TRANS_H
-#line 1 "declaration"
+#line 5 "edit-distance-int-trans.h"
 int distance_int_trans (
                     text_fuzzy_t * tf)
 ;
