@@ -8,23 +8,31 @@
 #include "config.h"
 #include "text-fuzzy.h"
 #include "edit-distance-int.h"
-#line 1 "declaration"
+#line 12 "edit-distance-int.c"
 int distance_int (
                     text_fuzzy_t * tf)
 
 {
-#line 98 "edit-distance.c.tmpl"
 
 
 
 
-#line 109 "edit-distance.c.tmpl"
+#line 127 "edit-distance.c.tmpl"
+#line 129 "edit-distance.c.tmpl"
+
     const unsigned int * word1 = (const unsigned int *) tf->b.unicode;
     int len1 = tf->b.ulength;
     const unsigned int * word2 = (const unsigned int *) tf->text.unicode;
     int len2 = tf->text.ulength;
-
-#line 208 "edit-distance.c.tmpl"
+    /* Return value. */
+    int d;
+    /* X and Y coordinates in the matrix of strings. */
+    int i;
+    int j;
+#line 142 "edit-distance.c.tmpl"
+    int max;
+#line 145 "edit-distance.c.tmpl"
+#line 271 "edit-distance.c.tmpl"
 
     /* Matrix is the dynamic programming matrix. We economize on space
        by having only two columns. */
@@ -33,21 +41,19 @@ int distance_int (
     int matrix[2][len2 + 1];
 #else
     int * matrix[2];
-    int d;
 #endif
-    int i;
-    int j;
     int large_value;
 
-#line 224 "edit-distance.c.tmpl"
-    int max;
-
     max = tf->max_distance;
-#line 229 "edit-distance.c.tmpl"
 
 #ifndef __GNUC__
     for (i = 0; i < 2; i++) {
 	matrix[i] = calloc (len2 + 1, sizeof (int));
+	if (! matrix[i]) {
+	    fprintf (stderr, "%s:%d: calloc failed.\n",
+		     __FILE__, __LINE__);
+	    return -1;
+	}
     }
 #endif
 
@@ -170,20 +176,18 @@ int distance_int (
             }
         }
     }
-#ifdef __GNUC__
-
-    return matrix[len1 % 2][len2];
-
-#else
     d = matrix[len1 % 2][len2];
 
+#ifndef __GNUC__
     for (i = 0; i < 2; i++) {
 	free (matrix[i]);
     }
-
-    return d;
 #endif
-
-#line 371 "edit-distance.c.tmpl"
+#line 425 "edit-distance.c.tmpl"
+    return d;
 }
 
+/* Some people are not using GCC-compatible compilers. See, for
+   example, this test failure.
+   http://ppm4.activestate.com/sun4-solaris-64/5.12/1200/B/BK/BKB/Text-Fuzzy-0.11.d/log-20130328T025706.txt */
+
