@@ -1,9 +1,9 @@
+#line 2 "edit-distance.h.tmpl"
 #ifndef EDIT_DISTANCE_INT_H
 #define EDIT_DISTANCE_INT_H
-
-int distance_int (const unsigned int * word1,
-                    int len1,
+#line 5 "edit-distance-int.h"
+int distance_int (
                     text_fuzzy_t * tf)
 ;
-
+#line 7 "edit-distance.h.tmpl"
 #endif /* EDIT_DISTANCE_INT_H */
