@@ -1,36 +1,38 @@
 #include <stdio.h>
-#include <string.h>
-#include <errno.h>
-#include <limits.h>
-#include "config.h"
-#include "text-fuzzy.h"
-#include "edit-distance-char-trans.h"
-#include "edit-distance-int-trans.h"
-#include "edit-distance-char.h"
-#include "edit-distance-int.h"
-
-#ifndef ERROR_HANDLER
-#define ERROR_HANDLER text_fuzzy_error_handler;
-#endif /* undef ERROR_HANDLER */
-#include "text-fuzzy.h"
-#include <stdio.h>
 #include <stdarg.h>
 #include <stdlib.h>
 
+#ifdef HEADER
+#ifndef ERROR_HANDLER
+#define ERROR_HANDLER text_fuzzy_error_handler
+#endif /* undef ERROR_HANDLER */
+
 #ifndef ERROR_HANDLER_H
 #define ERROR_HANDLER_H
+
 typedef int (* error_handler_t) (const char * source_file,
                                  int source_line_number,
                                  const char * message, ...)
-#ifdef __GNUC__
-    __attribute__ ((format (printf, 3, 4)))
-#endif /* __GNUC__ */
-;
+    __attribute__ ((format (printf, 3, 4)));
+
 #endif /* ndef ERROR_HANDLER_H */
 
 extern error_handler_t text_fuzzy_error_handler;
 
 
+#endif /* def HEADER */
+
+#ifndef ERROR_HANDLER_H
+#define ERROR_HANDLER_H
+
+typedef int (* error_handler_t) (const char * source_file,
+                                 int source_line_number,
+                                 const char * message, ...)
+    __attribute__ ((format (printf, 3, 4)));
+
+#endif /* ndef ERROR_HANDLER_H */
+
+
 /* This is the default error handler for this namespace. */
 
 static int
@@ -76,8 +78,6 @@ error_handler_t text_fuzzy_error_handler =
 
 /* Fail a test, with message. */
 
-#ifdef __GNUC__
-
 #define FAIL_MSG(condition, status, msg, args...)                       \
     if (condition) {                                                    \
         LINE_ERROR (condition, status);                                 \
@@ -89,19 +89,6 @@ error_handler_t text_fuzzy_error_handler =
         return text_fuzzy_status_ ## status;                      \
     }
 
-#else /* __GNUC__ */
-
-#define FAIL_MSG fail_msg
-
-static void fail_msg (int condition, int status, char * msg, ...)
-{
-    if (condition) {
-	fprintf (stderr, "%d:%s", status, msg);
-    }
-}
-
-#endif /* __GNUC__ */
-
 #define OK return text_fuzzy_status_ok;
 
 /* Call a function and print an error message and return if the
@@ -131,6 +118,7 @@ End:
 const char * text_fuzzy_statuses[] = {
     "normal operation",
     "out of memory",
+    "a pointer contained a zero value",
     "open error",
     "close error",
     "read error",
@@ -142,10 +130,6 @@ const char * text_fuzzy_statuses[] = {
     "miscount",
 };
 
-#define STATIC static
-#define FUNC(name) text_fuzzy_status_t text_fuzzy_ ## name
-
-#ifdef __GNUC__
 #ifdef VERBOSE
 #define MESSAGE(format, args...) {              \
         printf ("%s:%d: ", __FILE__, __LINE__); \
@@ -154,59 +138,96 @@ const char * text_fuzzy_statuses[] = {
 #else /* VERBOSE */
 #define MESSAGE(format, args...)
 #endif /* VERBOSE */
-#else /* __GNUC__ */
-#define MESSAGE empty_message
-static void empty_message (const char * format, ...) { return; }
-#endif /* __GNUC__ */
 
 /* Local variables:
 mode: c
 End:
 */
 
-static int n_mallocs
-#ifdef __GNUC__
-__attribute__ ((unused))
-#endif /* def __GNUC__ */
-;
+#ifdef HEADER
+extern const char * text_fuzzy_statuses[];
+#ifndef ERROR_HANDLER_H
+#define ERROR_HANDLER_H
 
-#define ALLOC(x,n) {				\
-	x = malloc (sizeof (* x) * n);		\
-	n_mallocs++;				\
-	FAIL(!x, memory_failure);		\
+typedef int (* error_handler_t) (const char * source_file,
+                                 int source_line_number,
+                                 const char * message, ...)
+    __attribute__ ((format (printf, 3, 4)));
+
+#endif /* ndef ERROR_HANDLER_H */
+
+extern error_handler_t text_fuzzy_error_handler;
+
+#ifndef ERROR_HANDLER
+
+#include <stdio.h>
+#include <stdarg.h>
+
+static void default_error_handler (const char * file, int line,
+                                   const char * format, ...)
+{
+    va_list a;
+    va_start (a, format);
+    fprintf (stderr, "%s:%d ", file, line);
+    vfprintf (stderr, format, a);
+    fprintf (stderr, "\n");
+    va_end (a);
+}
+#define ERROR_HANDLER default_error_handler
+#endif /* ERROR_HANDLER */
+#define TEXT_FUZZY(x) {                                                 \
+    text_fuzzy_status_t status;                                   \
+    status = text_fuzzy_ ## x;                                    \
+    if (status != text_fuzzy_status_ok) {                         \
+    /* Print error and return. */                                       \
+    ERROR_HANDLER (__FILE__, __LINE__,                                  \
+                   "Call to %s failed: %s",                             \
+                   #x, text_fuzzy_statuses[status]);              \
+    return TEXT_FUZZY_USER_ERROR;					\
+    }                                                                   \
     }
 
-#define ZALLOC(x,n) {				\
-	x = calloc (n, sizeof (* x));		\
-	n_mallocs++;				\
-	FAIL(!x, memory_failure);		\
-    }
+/*
+  Local variables:
+  mode: c
+  End: 
+*/
 
-#define FREE(x) {				\
-	free (x);				\
-	x = 0;					\
-	n_mallocs--;				\
-    }
+typedef enum {
+    text_fuzzy_status_ok,
+    text_fuzzy_status_memory_failure,
+    text_fuzzy_status_null_pointer,
+    text_fuzzy_status_open_error,
+    text_fuzzy_status_close_error,
+    text_fuzzy_status_read_error,
+    text_fuzzy_status_line_too_long,
+    text_fuzzy_status_ualphabet_on_non_unicode,
+    text_fuzzy_status_max_min_miscalculation,
+    text_fuzzy_status_string_too_long,
+    text_fuzzy_status_max_distance_misuse,
+    text_fuzzy_status_miscount,
+}
+text_fuzzy_status_t;
+
+
+#endif /* def HEADER */
 
 #line 1 "/usr/home/ben/projects/text-fuzzy/text-fuzzy.c.in"
+#include <stdio.h>
+#include <string.h>
+#include <errno.h>
+#include <limits.h>
 
-
-
-/* For error-handling for the file opening functions. */
-
-/* For INT_MAX, INT_MIN. */
-
-
-
-
+#include "config.h"
+#include "text-fuzzy.h"
 
 /* All of the following are automatically generated from
    "edit-distance.h.tmpl" by "make-edit-distance-c.pl". */
 
-
-
-
-
+#include "edit-distance-char-trans.h"
+#include "edit-distance-int-trans.h"
+#include "edit-distance-char.h"
+#include "edit-distance-int.h"
 
 /* The following starts off the header file. This is a tag which tells
    "C::Maker" to start writing "text-fuzzy.h". */
@@ -384,7 +405,7 @@ text_fuzzy_t;
 
 /* Generate the Unicode alphabet in "tf->ualphabet". */
 
-FUNC (generate_ualphabet) (text_fuzzy_t * tf)
+text_fuzzy_status_t text_fuzzy_generate_ualphabet (text_fuzzy_t * tf)
 {
     int i;
 
@@ -570,7 +591,7 @@ static int ualphabet_miss (text_fuzzy_t * tf, text_fuzzy_string_t * b)
    code for the dynamic programming algorithms is in
    "edit-distance.c.tmpl". */
 
-FUNC (compare_single) (text_fuzzy_t * tf)
+text_fuzzy_status_t text_fuzzy_compare_single (text_fuzzy_t * tf)
 {
 
     /* The edit distance between "tf->search_term" and the
@@ -740,7 +761,7 @@ FUNC (compare_single) (text_fuzzy_t * tf)
     OK;
 }
 
-FUNC (get_candidates) (text_fuzzy_t * text_fuzzy,
+text_fuzzy_status_t text_fuzzy_get_candidates (text_fuzzy_t * text_fuzzy,
 		       int * n_candidates_ptr,
 		       int ** candidates_ptr)
 {
@@ -798,7 +819,7 @@ FUNC (get_candidates) (text_fuzzy_t * text_fuzzy,
     OK;
 }
 
-FUNC (free_candidates) (text_fuzzy_t * text_fuzzy, int * candidates)
+text_fuzzy_status_t text_fuzzy_free_candidates (text_fuzzy_t * text_fuzzy, int * candidates)
 {
     if (candidates) {
 	free (candidates);
@@ -818,7 +839,7 @@ static int max_unique_characters = 45;
    non-matching terms without using the dynamic programming
    algorithm. */
 
-FUNC (generate_alphabet) (text_fuzzy_t * text_fuzzy)
+text_fuzzy_status_t text_fuzzy_generate_alphabet (text_fuzzy_t * text_fuzzy)
 {
     int unique_characters;
     int i;
@@ -852,7 +873,7 @@ FUNC (generate_alphabet) (text_fuzzy_t * text_fuzzy)
     OK;
 }
 
-FUNC (begin_scanning) (text_fuzzy_t * text_fuzzy)
+text_fuzzy_status_t text_fuzzy_begin_scanning (text_fuzzy_t * text_fuzzy)
 {
     /* Even if the user does not want to set a maximum distance, set
        one anyway so that we can reject stuff without going into the
@@ -887,7 +908,7 @@ FUNC (begin_scanning) (text_fuzzy_t * text_fuzzy)
 /* Put the user's desired maximum distance back into the object for
    the next search. */
 
-FUNC (end_scanning) (text_fuzzy_t * text_fuzzy)
+text_fuzzy_status_t text_fuzzy_end_scanning (text_fuzzy_t * text_fuzzy)
 {
     text_fuzzy->max_distance = text_fuzzy->max_distance_holder;
     text_fuzzy->scanning = 0;
@@ -921,7 +942,7 @@ fuzzy_file_t;
 
 #define SIZE 0x1000
 
-STATIC FUNC (more_bytes) (fuzzy_file_t * ff)
+static text_fuzzy_status_t text_fuzzy_more_bytes (fuzzy_file_t * ff)
 {
     int bytes;
 
@@ -939,7 +960,7 @@ STATIC FUNC (more_bytes) (fuzzy_file_t * ff)
     OK;
 }
 
-STATIC FUNC (get_line) (fuzzy_file_t * ff)
+static text_fuzzy_status_t text_fuzzy_get_line (fuzzy_file_t * ff)
 {
     int i;
     static char s[SIZE];
@@ -970,7 +991,7 @@ STATIC FUNC (get_line) (fuzzy_file_t * ff)
     OK;
 }
 
-STATIC FUNC (open) (fuzzy_file_t * ff, const char * file_name)
+static text_fuzzy_status_t text_fuzzy_open (fuzzy_file_t * ff, const char * file_name)
 {
     ff->file_name = file_name;
     ff->fh = fopen (ff->file_name, "r");
@@ -979,7 +1000,7 @@ STATIC FUNC (open) (fuzzy_file_t * ff, const char * file_name)
     OK;
 }
 
-STATIC FUNC (close) (fuzzy_file_t * ff)
+static text_fuzzy_status_t text_fuzzy_close (fuzzy_file_t * ff)
 {
     FAIL (fclose (ff->fh), close_error);
     OK;
@@ -988,7 +1009,7 @@ STATIC FUNC (close) (fuzzy_file_t * ff)
 /* Scan the file specified by "file_name" for our string. The nearest
    string found is returned in "* nearest_ptr". */
 
-FUNC (scan_file) (text_fuzzy_t * text_fuzzy, char * file_name,
+text_fuzzy_status_t text_fuzzy_scan_file (text_fuzzy_t * text_fuzzy, char * file_name,
                   char ** nearest_ptr, int * nearest_length_ptr)
 {
     fuzzy_file_t ff = {0};
@@ -1036,13 +1057,13 @@ FUNC (scan_file) (text_fuzzy_t * text_fuzzy, char * file_name,
     OK;
 }
 
-FUNC (scan_file_free) (char * nearest)
+text_fuzzy_status_t text_fuzzy_scan_file_free (char * nearest)
 {
     free (nearest);
     OK;
 }
 
-FUNC (alphabet_rejections) (text_fuzzy_t * text_fuzzy, int * r)
+text_fuzzy_status_t text_fuzzy_alphabet_rejections (text_fuzzy_t * text_fuzzy, int * r)
 {
     * r = text_fuzzy->alphabet_rejections;
     OK;
@@ -1052,7 +1073,7 @@ FUNC (alphabet_rejections) (text_fuzzy_t * text_fuzzy, int * r)
    about "free to wrong pool". See
    "http://www.perlmonks.org/?node_id=742205" */
 
-FUNC (free_memory) (text_fuzzy_t * text_fuzzy)
+text_fuzzy_status_t text_fuzzy_free_memory (text_fuzzy_t * text_fuzzy)
 {
     if (text_fuzzy->ualphabet.alphabet) {
 	free (text_fuzzy->ualphabet.alphabet);
@@ -1061,37 +1082,37 @@ FUNC (free_memory) (text_fuzzy_t * text_fuzzy)
     OK;
 }
 
-FUNC (set_max_distance) (text_fuzzy_t * text_fuzzy, int max_distance)
+text_fuzzy_status_t text_fuzzy_set_max_distance (text_fuzzy_t * text_fuzzy, int max_distance)
 {
     text_fuzzy->max_distance = max_distance;
     OK;
 }
 
-FUNC (get_max_distance) (text_fuzzy_t * text_fuzzy, int * max_distance)
+text_fuzzy_status_t text_fuzzy_get_max_distance (text_fuzzy_t * text_fuzzy, int * max_distance)
 {
     * max_distance = text_fuzzy->max_distance;
     OK;
 }
 
-FUNC (set_transpositions) (text_fuzzy_t * text_fuzzy, int transpositions)
+text_fuzzy_status_t text_fuzzy_set_transpositions (text_fuzzy_t * text_fuzzy, int transpositions)
 {
     text_fuzzy->transpositions_ok = transpositions != 0 ? 1 : 0;
     OK;
 }
 
-FUNC (get_transpositions) (text_fuzzy_t * text_fuzzy, int * transpositions)
+text_fuzzy_status_t text_fuzzy_get_transpositions (text_fuzzy_t * text_fuzzy, int * transpositions)
 {
     * transpositions = text_fuzzy->transpositions_ok;
     OK;
 }
 
-FUNC (last_distance) (text_fuzzy_t * text_fuzzy, int * last_distance)
+text_fuzzy_status_t text_fuzzy_last_distance (text_fuzzy_t * text_fuzzy, int * last_distance)
 {
     * last_distance = text_fuzzy->distance;
     OK;
 }
 
-FUNC (no_alphabet) (text_fuzzy_t * text_fuzzy, int yes_no)
+text_fuzzy_status_t text_fuzzy_no_alphabet (text_fuzzy_t * text_fuzzy, int yes_no)
 {
     text_fuzzy->user_no_alphabet = yes_no != 0 ? 1 : 0;
     if (text_fuzzy->user_no_alphabet) {
@@ -1101,25 +1122,25 @@ FUNC (no_alphabet) (text_fuzzy_t * text_fuzzy, int yes_no)
     OK;
 }
 
-FUNC (ualphabet_rejections) (text_fuzzy_t * text_fuzzy, int * ualphabet_rejections)
+text_fuzzy_status_t text_fuzzy_ualphabet_rejections (text_fuzzy_t * text_fuzzy, int * ualphabet_rejections)
 {
     * ualphabet_rejections = text_fuzzy->ualphabet.rejections;
     OK;
 }
 
-FUNC (set_no_exact) (text_fuzzy_t * text_fuzzy, int yes_no)
+text_fuzzy_status_t text_fuzzy_set_no_exact (text_fuzzy_t * text_fuzzy, int yes_no)
 {
     text_fuzzy->no_exact = yes_no != 0 ? 1 : 0;
     OK;
 }
 
-FUNC (get_length_rejections) (text_fuzzy_t * text_fuzzy, int * length_rejections)
+text_fuzzy_status_t text_fuzzy_get_length_rejections (text_fuzzy_t * text_fuzzy, int * length_rejections)
 {
     * length_rejections = text_fuzzy->length_rejections;
     OK;
 }
 
-FUNC (get_unicode_length) (text_fuzzy_t * text_fuzzy, int * unicode_length)
+text_fuzzy_status_t text_fuzzy_get_unicode_length (text_fuzzy_t * text_fuzzy, int * unicode_length)
 {
     if (text_fuzzy->text.unicode) {
 	* unicode_length = text_fuzzy->text.ulength;
@@ -1161,13 +1182,3 @@ status: miscount
 
 */
 
-
-void
-text_fuzzy_check_memory ()
-{
-    if (n_mallocs != 0) {
-        (*text_fuzzy_error_handler) (__FILE__, __LINE__,
-	                                   "n_mallocs = %d", n_mallocs);
-    }
-}
-
